package org.nakedobjects.nos.store.file;

import junit.framework.TestCase;

import org.apache.log4j.Level;
import org.apache.log4j.LogManager;
import org.nakedobjects.noa.reflect.NakedObjectField;
import org.nakedobjects.nof.core.adapter.SerialNumberVersion;
import org.nakedobjects.nof.core.context.NakedObjectsContext;
import org.nakedobjects.nof.core.context.StaticContext;
import org.nakedobjects.nof.persist.transaction.PersistenceCommand;
import org.nakedobjects.testing.DummyNakedObject;
import org.nakedobjects.testing.NullSession;
import org.nakedobjects.testing.TestConfiguration;
import org.nakedobjects.testing.TestObjectLoader;
import org.nakedobjects.testing.TestPersistor;
import org.nakedobjects.testing.TestSpecification;
import org.nakedobjects.testing.TestSpecificationLoader;



public class XmlObjectStoreTest extends TestCase {
    private XmlObjectStore objectStore;
    private MockDataManager dataManager;
    private DummyNakedObject nakedObject;
    private TestSpecification spec;

    public static void main(final String[] args) {
        junit.textui.TestRunner.run(XmlObjectStoreTest.class);
    }

    protected void setUp() throws Exception {
        LogManager.getLoggerRepository().setThreshold(Level.OFF);

        // system
        dataManager = new MockDataManager();
        objectStore = new XmlObjectStore(dataManager, new DummyServiceManager());
        objectStore.setClock(new TestClock());

        NakedObjectsContext system = StaticContext.createInstance();
        NakedObjectsContext.setConfiguration(new TestConfiguration());
        system.setReflector(new TestSpecificationLoader());
        system.setObjectPersistor(new TestPersistor());
        system.setObjectLoader(new TestObjectLoader());
        system.setSession(new NullSession());

        // objects
        spec = new TestSpecification();
        spec.fields = new NakedObjectField[0];
        nakedObject = new DummyNakedObject();
        nakedObject.setupSpecification(spec);
        nakedObject.setOptimisticLock(new SerialNumberVersion(23, null, null));
    }

    public void testSaveObjectCreatesNewVersion() throws Exception {
        nakedObject.setOptimisticLock(null);

        PersistenceCommand[] commands = new PersistenceCommand[1];
        commands[0] = objectStore.createCreateObjectCommand(nakedObject);
        objectStore.execute(commands);

        assertEquals(new FileVersion(null, 1), nakedObject.getVersion());
    }

    public void testDeleteObjectRemovesVersion() throws Exception {
        PersistenceCommand[] commands = new PersistenceCommand[1];
        commands[0] = objectStore.createDestroyObjectCommand(nakedObject);
        objectStore.execute(commands);

        assertEquals(null, nakedObject.getVersion());
    }

    public void testUpdateObjectCreatesNewVersion() throws Exception {
        PersistenceCommand[] commands = new PersistenceCommand[1];
        commands[0] = objectStore.createSaveObjectCommand(nakedObject);
        objectStore.execute(commands);

        assertEquals(new FileVersion(null, 1), nakedObject.getVersion());
    }
}
// Copyright (c) Naked Objects Group Ltd.
